@@ -11,9 +11,18 @@ use Marpa::R2;
 use Log::Any qw/$log/;
 use Carp qw/croak/;
 
-our $VERSION = '0.001'; # VERSION
+our $VERSION = '0.002'; # VERSION
 
 
+#
+# List of escaped characters allowed in terminfo source files
+# ^x : Control-x (for any appropriate x)
+# \x where x can be a b E e f l n r s t ^ \ , : 0
+#
+our $CONTROLX      = qr/(?<!\^)(?>\^\^)*\^./;                                                       # Takes care of ^^
+our $ALLOWED_BACKSLASHED_CHARACTERS = qr/(?:a|b|E|e|f|l|n|r|s|t|\^|\\|,|:|0|\d{3})/;
+our $BACKSLASHX    = qr/(?<!\\)(?>\\\\)*\\$ALLOWED_BACKSLASHED_CHARACTERS/;                         # Takes care of \\
+our $ESCAPED       = qr/(?:$CONTROLX|$BACKSLASHX)/;
 our $I_CONSTANT = qr/(?:(0[xX][a-fA-F0-9]+(?:[uU](?:ll|LL|[lL])?|(?:ll|LL|[lL])[uU]?)?)             # Hexadecimal
                       |([1-9][0-9]*(?:[uU](?:ll|LL|[lL])?|(?:ll|LL|[lL])[uU]?)?)                    # Decimal
                       |(0[0-7]*(?:[uU](?:ll|LL|[lL])?|(?:ll|LL|[lL])[uU]?)?)                        # Octal
@@ -23,17 +32,22 @@ our $I_CONSTANT = qr/(?:(0[xX][a-fA-F0-9]+(?:[uU](?:ll|LL|[lL])?|(?:ll|LL|[lL])[
 #
 # It is important to have LONGNAME before ALIAS because LONGNAME will do a lookahead on COMMA
 # It is important to have NUMERIC and STRING before BOOLEAN because BOOLEAN is a subset of them
+# It is important to have BLANKLINE and COMMENT at the end: they are 'discarded' by the grammar
+# In these regexps we add the embedded comma: \, (i.e. these are TWO characters)
 #
 our @TOKENSRE = (
-    [ 'ALIASINCOLUMNONE' , qr/\G^(\p{MarpaX::Database::Terminfo::Grammar::CharacterClasses::InAlias}+)/ ],
+    [ 'ALIASINCOLUMNONE' , qr/(?:\A|\n)\G((?:$ESCAPED|\p{MarpaX::Database::Terminfo::Grammar::CharacterClasses::InAlias})+)/ ],
     [ 'PIPE'             , qr/\G(\|)/ ],
-    [ 'LONGNAME'         , qr/\G(\p{MarpaX::Database::Terminfo::Grammar::CharacterClasses::InLongname}+), ?/ ],
-    [ 'ALIAS'            , qr/\G(\p{MarpaX::Database::Terminfo::Grammar::CharacterClasses::InAlias}+)/ ],
-    [ 'NUMERIC'          , qr/\G(\p{MarpaX::Database::Terminfo::Grammar::CharacterClasses::InName}+#$I_CONSTANT)/ ],
-    [ 'STRING'           , qr/\G(\p{MarpaX::Database::Terminfo::Grammar::CharacterClasses::InName}+=\p{MarpaX::Database::Terminfo::Grammar::CharacterClasses::InIsPrintExceptComma}+)/ ],
-    [ 'BOOLEAN'          , qr/\G(\p{MarpaX::Database::Terminfo::Grammar::CharacterClasses::InName}+)/ ],
+    [ 'LONGNAME'         , qr/\G((?:$ESCAPED|\p{MarpaX::Database::Terminfo::Grammar::CharacterClasses::InNcursesLongname})+), ?/ ],
+    [ 'ALIAS'            , qr/\G((?:$ESCAPED|\p{MarpaX::Database::Terminfo::Grammar::CharacterClasses::InAlias})+)/ ],
+    [ 'NUMERIC'          , qr/\G((?:$ESCAPED|\p{MarpaX::Database::Terminfo::Grammar::CharacterClasses::InName})+#$I_CONSTANT)/ ],
+    [ 'STRING'           , qr/\G((?:$ESCAPED|\p{MarpaX::Database::Terminfo::Grammar::CharacterClasses::InName})+=(?:$ESCAPED|\p{MarpaX::Database::Terminfo::Grammar::CharacterClasses::InIsPrintExceptComma})*)/ ],
+    [ 'BOOLEAN'          , qr/\G((?:$ESCAPED|\p{MarpaX::Database::Terminfo::Grammar::CharacterClasses::InName})+)/ ],
     [ 'COMMA'            , qr/\G(, ?)/ ],
+    [ 'NEWLINE'          , qr/\G(\n)/ ],
     [ 'WS_many'          , qr/\G( +)/ ],
+    [ 'BLANKLINE'        , qr/\G(?:\A|\n)([ \t]*\n)/ ],
+    [ 'COMMENT'          , qr/\G(?:\A|\n)([ \t]*#[^\n]*\n)/ ],
     );
 
 my %events = (
@@ -44,23 +58,31 @@ my %events = (
 	my $prev = pos(${$bufferp});
 	pos(${$bufferp}) = $start;
 	my $ok = 0;
-	if ($log->is_debug) {
-	    $log->debugf('Expected terminals: %s', \@expected);
+	if ($log->is_trace) {
+	    $log->tracef('Expected terminals: %s', \@expected);
 	}
 	foreach (@TOKENSRE) {
 	    my ($token, $re) = @{$_};
-	    if ((grep {$_ eq $token} @expected) && ${$bufferp} =~ $re) {
-		$length = $+[1] - $-[1];
-		$string = substr(${$bufferp}, $start, $length);
-		if ($log->is_debug) {
-		    $log->debugf('lexeme_read(\'%s\', %d, %d, \"%s\")', $token, $start, $length, $string);
+	    if ((grep {$_ eq $token} @expected)) {
+		if (${$bufferp} =~ $re) {
+		    $length = $+[1] - $-[1];
+		    $string = substr(${$bufferp}, $start, $length);
+		    if ($log->is_debug && $token eq 'LONGNAME') {
+			$log->debugf('%s "%s")', $token, $string);
+		    } elsif ($log->is_trace) {
+			$log->tracef('lexeme_read(token=%s, start=%d, length=%d, string="%s")', $token, $start, $length, $string);
+		    }
+		    $recce->lexeme_read($token, $start, $length, $string);
+		    $ok = 1;
+		    last;
+		} else {
+		    if ($log->is_trace) {
+			$log->tracef('\"%s\"... does not match %s', substr(${$bufferp}, $start, 20), $re);
+		    }
 		}
-		$recce->lexeme_read($token, $start, $length, $string);
-		$ok = 1;
-		last;
 	    }
 	}
-	die "Unmatched token in @expected" if (! $ok);
+	die "Unmatched token in @expected, current portion of string is \"$string\"" if (! $ok);
 	pos(${$bufferp}) = $prev;
     },
 );
@@ -131,6 +153,7 @@ sub value {
     return $rc
 }
 
+
 1;
 
 __END__
@@ -145,7 +168,7 @@ MarpaX::Database::Terminfo - Parse a terminfo data base using Marpa
 
 =head1 VERSION
 
-version 0.001
+version 0.002
 
 =head1 SYNOPSIS
 
@@ -178,7 +201,7 @@ version 0.001
 
 This module parses a terminfo database and produces an AST from it. If you want to enable logging, be aware that this module is a Log::Any thingy.
 
-The grammar is the one found at L<http://nixdoc.net/man-pages/HP-UX/man4/terminfo.4.html#Formal%20Grammar>.
+The grammar is a slightly revisited version of the one found at L<http://nixdoc.net/man-pages/HP-UX/man4/terminfo.4.html#Formal%20Grammar>, taking into account ncurses compatibility.
 
 =head1 SUBROUTINES/METHODS
 
@@ -194,6 +217,12 @@ Parses a terminfo database. Takes a pointer to a string as parameter.
 
 Returns Marpa's value on the parse tree. Ambiguous parse tree result is disabled and the module will croak if this happen.
 
+=head1 SEE ALSO
+
+L<Unix Documentation Project - terminfo|http://nixdoc.net/man-pages/HP-UX/man4/terminfo.4.html#Formal%20Grammar>
+
+L<GNU Ncurses|http://www.gnu.org/software/ncurses/>
+
 =for :stopwords cpan testmatrix url annocpan anno bugtracker rt cpants kwalitee diff irc mailto metadata placeholders metacpan
 
 =head1 SUPPORT
