@@ -5,7 +5,7 @@ package MarpaX::Database::Terminfo::Grammar::CharacterClasses;
 
 # ABSTRACT: Terminfo character classes
 
-our $VERSION = '0.001'; # VERSION
+our $VERSION = '0.002'; # VERSION
 
 
 our $COMMA_HEX  = sprintf('%x', ord(','));
@@ -29,6 +29,12 @@ $PIPE_HEX
 END
 }
 
+sub InPipe {
+    return <<END;
+$PIPE_HEX
+END
+}
+
 sub InCommaEqualPound {
     return <<END;
 $COMMA_HEX
@@ -57,6 +63,13 @@ sub InLongname {
 END
 }
 
+sub InNcursesLongname {
+    return <<END;
++utf8::IsPrint
+-MarpaX::Database::Terminfo::Grammar::CharacterClasses::InPipe
+END
+}
+
 sub InName {
     return <<END;
 +utf8::IsPrint
@@ -92,7 +105,7 @@ MarpaX::Database::Terminfo::Grammar::CharacterClasses - Terminfo character class
 
 =head1 VERSION
 
-version 0.001
+version 0.002
 
 =head1 DESCRIPTION
 
